"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.S3Thumbnail = void 0;
var util = __importStar(require("./util"));
var s3_1 = __importDefault(require("aws-sdk/clients/s3"));
var path_1 = require("path");
var url_1 = require("url");
;
var Thumbnail = /** @class */ (function () {
    function Thumbnail(input, config) {
        this.config = {
            prefix: "thumbnails/",
            width: 180,
            height: 180,
            time: "00:00:05",
            type: "jpg",
            quality: 2
        };
        this.generated = false;
        this.fileName = "";
        this.input = input;
        this.config = __assign(__assign({}, this.config), config);
        this.setType(this.config.type || "jpg");
        if (this.config.quality > 10)
            this.config.quality = 10;
        if (this.config.quality < 1)
            this.config.quality = 1;
        if (input !== "") {
            this.parseInput(input);
        }
        else {
            this.fileName = "thumbnail";
        }
    }
    Thumbnail.prototype.parseInput = function (input) {
        if (input.toLowerCase().match(/(http[s]?:\/\/|\/\/)/)) {
            var fileDetails = url_1.parse(input);
            this.fileName = path_1.posix.basename(fileDetails.pathname || "");
        }
        else {
            var fileDetails = path_1.parse(input);
            this.config.path = fileDetails.dir;
            this.fileName = fileDetails.name;
        }
    };
    Thumbnail.prototype.getOutput = function () {
        var output;
        var path = (this.config.path || "");
        if (path !== "")
            path += "/";
        output = path + (this.config.prefix || "") + util.getFileName(this.fileName) + (this.config.suffix || "");
        if (this.getType().toLowerCase().match(/jpg|jpeg/))
            output += ".jpg";
        if (this.getType().toLowerCase() === "png")
            output += ".png";
        if (this.getType().toLowerCase() === "webp")
            output += ".webp";
        return output;
    };
    Thumbnail.prototype.isGenerated = function () {
        return this.generated;
    };
    Thumbnail.prototype.setTime = function (time) {
        if (!time.match(/\d\d:\d\d:\d\d/))
            throw new ThumbnailException("Invalid time format must match format 00:00:00");
        this.config.time = time;
    };
    Thumbnail.prototype.getTime = function () {
        return this.config.time || "00:00:10";
    };
    Thumbnail.prototype.setType = function (type) {
        if (!type.toLowerCase().match(/jpg|jpeg|png|webp/))
            throw new ThumbnailException("Invalid type given. Valid types are jpg, png or webp");
        this.type = type;
    };
    Thumbnail.prototype.getType = function () {
        return this.type || "jpg";
    };
    Thumbnail.prototype.getInput = function () {
        return this.input;
    };
    Thumbnail.prototype.getFFmpegConfig = function () {
        var config = {
            quality: this.config.quality,
            codec: "mjpeg",
            filter: "image2pipe",
            timestamp: "00:00:10",
            width: this.config.width,
            height: this.config.height
        };
        if (this.getType().toLowerCase() === "png")
            config.codec = "png"; //default is png
        if (this.getType().toLowerCase() === "webp") {
            config.codec = "libwebp";
            config.quality = (11 - config.quality) * 10;
        }
        return config;
    };
    Thumbnail.prototype.getConfig = function () {
        return this.config;
    };
    return Thumbnail;
}());
exports.default = Thumbnail;
var S3Thumbnail = /** @class */ (function (_super) {
    __extends(S3Thumbnail, _super);
    function S3Thumbnail(bucket, key, config) {
        var _this = _super.call(this, "", config) || this;
        _this.bucket = _this.config.outputBucket || bucket;
        _this.originalBucket = bucket;
        _this.key = key;
        _this.fileName = util.getNewName(_this.config.outputKey || _this.key, _this.getType());
        return _this;
    }
    S3Thumbnail.prototype.getS3Url = function () {
        var s3 = new s3_1.default();
        return s3.getSignedUrl("getObject", { Bucket: this.originalBucket, Key: this.key, Expires: 1100 });
    };
    S3Thumbnail.prototype.getInput = function () {
        return this.getS3Url();
    };
    S3Thumbnail.prototype.getBucket = function () {
        return this.bucket;
    };
    S3Thumbnail.prototype.getKey = function () {
        return this.key;
    };
    return S3Thumbnail;
}(Thumbnail));
exports.S3Thumbnail = S3Thumbnail;
var ThumbnailException = /** @class */ (function (_super) {
    __extends(ThumbnailException, _super);
    function ThumbnailException() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return ThumbnailException;
}(Error));
